@@ -28,8 +28,11 @@ usage(void)
 	fprintf(stderr, "Ldns notify utility\n\n");
 	fprintf(stderr, " Supported options:\n");
 	fprintf(stderr, "\t-z zone\t\tThe zone\n");
+	fprintf(stderr, "\t-I <address>\tsource address to query from\n");
 	fprintf(stderr, "\t-s version\tSOA version number to include\n");
-	fprintf(stderr, "\t-y key:data\tTSIG sign the query\n");
+	fprintf(stderr, "\t-y <name:key[:algo]>\tspecify named base64 tsig key"
+	                ", and optional an\n\t\t\t"
+	                "algorithm (defaults to hmac-md5.sig-alg.reg.int)\n");
 	fprintf(stderr, "\t-p port\t\tport to use to send to\n");
 	fprintf(stderr, "\t-v\t\tPrint version information\n");
 	fprintf(stderr, "\t-d\t\tPrint verbose debug information\n");
@@ -171,22 +174,23 @@ main(int argc, char **argv)
 	/* LDNS types */
 	ldns_pkt *notify;
 	ldns_rr *question;
-	ldns_resolver *res;
 	ldns_rdf *ldns_zone_name = NULL;
 	ldns_status status;
 	const char *zone_name = NULL;
 	int include_soa = 0;
 	uint32_t soa_version = 0;
-	ldns_tsig_credentials tsig_cred = {0,0,0};
 	int do_hexdump = 1;
 	uint8_t *wire = NULL;
 	size_t wiresize = 0;
 	const char *port = "53";
-	char *keydata;
+	char *tsig_sep;
+	const char *tsig_name = NULL, *tsig_data = NULL, *tsig_algo = NULL;
+	int error;
+	struct addrinfo from_hints, *from0 = NULL;
 
 	srandom(time(NULL) ^ getpid());
 
-        while ((c = getopt(argc, argv, "vhdp:r:s:y:z:")) != -1) {
+        while ((c = getopt(argc, argv, "vhdp:r:s:y:z:I:")) != -1) {
                 switch (c) {
                 case 'd':
 			verbose++;
@@ -202,18 +206,52 @@ main(int argc, char **argv)
 			soa_version = (uint32_t)atoi(optarg);
 			break;
                 case 'y':
-			tsig_cred.algorithm = (char*)"hmac-md5.sig-alg.reg.int.";
-			tsig_cred.keyname = optarg;
-			keydata = strchr(optarg, ':');
-			if (keydata == NULL) {
+			if (!(tsig_sep = strchr(optarg, ':'))) {
 				printf("TSIG argument is not in form "
-					"key:data: %s\n", optarg);
+					"<name:key[:algo]> %s\n", optarg);
 				exit(1);
 			}
-			*keydata++ = '\0';
-			tsig_cred.keydata = keydata;
-			printf("Sign with %s : %s\n", tsig_cred.keyname,
-				tsig_cred.keydata);
+			tsig_name = optarg;
+			*tsig_sep++ = '\0';
+			tsig_data = tsig_sep;
+			if ((tsig_sep = strchr(tsig_sep, ':'))) {
+				*tsig_sep++ = '\0';
+				tsig_algo = tsig_sep;
+			} else {
+				tsig_algo = "hmac-md5.sig-alg.reg.int.";
+			}
+			/* With dig TSIG keys are also specified with -y,
+			 * but format with drill is: -y <name:key[:algo]>
+			 *             and with dig: -y [hmac:]name:key
+			 *
+			 * When we detect an unknown tsig algorithm in algo,
+			 * but a known algorithm in name, we cane assume dig
+			 * order was used.
+			 *
+			 * Following if statement is to anticipate and correct
+			 * dig order
+			 */
+			if (strcasecmp(tsig_algo, "hmac-md5.sig-alg.reg.int")&&
+			    strcasecmp(tsig_algo, "hmac-md5")                &&
+			    strcasecmp(tsig_algo, "hmac-sha1")               &&
+			    strcasecmp(tsig_algo, "hmac-sha256")             &&
+			    strcasecmp(tsig_algo, "hmac-sha384")             &&
+			    strcasecmp(tsig_algo, "hmac-sha512")             &&
+			    ! (strcasecmp(tsig_name, "hmac-md5.sig-alg.reg.int")
+			    && strcasecmp(tsig_name, "hmac-md5")
+			    && strcasecmp(tsig_name, "hmac-sha1")
+			    && strcasecmp(tsig_name, "hmac-sha256")
+			    && strcasecmp(tsig_name, "hmac-sha384")
+			    && strcasecmp(tsig_name, "hmac-sha512"))) {
+
+				/* Roll options */
+				const char *tmp_tsig_algo = tsig_name;
+				tsig_name = tsig_data;
+				tsig_data = tsig_algo;
+				tsig_algo = tmp_tsig_algo;
+			}	
+			printf("Sign with name: %s, data: %s, algorithm: %s\n"
+			      , tsig_name, tsig_data, tsig_algo);
 			break;
                 case 'z':
 			zone_name = optarg;
@@ -224,8 +262,22 @@ main(int argc, char **argv)
 				exit(1);
 			}
                         break;
+		case 'I':
+			memset(&from_hints, 0, sizeof(from_hints));
+			from_hints.ai_family = AF_UNSPEC;
+			from_hints.ai_socktype = SOCK_DGRAM;
+			from_hints.ai_protocol = IPPROTO_UDP;
+			from_hints.ai_flags = AI_NUMERICHOST;
+			error = getaddrinfo(optarg, 0, &from_hints, &from0);
+			if (error) {
+				printf("bad address: %s: %s\n", optarg,
+					gai_strerror(error));
+				exit(EXIT_FAILURE);
+			}
+			break;
 		case 'v':
 			version();
+			/* fallthrough */
                 case 'h':
                 case '?':
                 default:
@@ -241,9 +293,8 @@ main(int argc, char **argv)
 
 	notify = ldns_pkt_new();
 	question = ldns_rr_new();
-	res = ldns_resolver_new();
 
-	if (!notify || !question || !res) {
+	if (!notify || !question) {
 		/* bail out */
 		printf("error: cannot create ldns types\n");
 		exit(1);
@@ -253,6 +304,7 @@ main(int argc, char **argv)
 	ldns_rr_set_class(question, LDNS_RR_CLASS_IN);
 	ldns_rr_set_owner(question, ldns_zone_name);
 	ldns_rr_set_type(question, LDNS_RR_TYPE_SOA);
+	ldns_rr_set_question(question, true);
 	ldns_pkt_set_opcode(notify, LDNS_PACKET_NOTIFY);
 	ldns_pkt_push_rr(notify, LDNS_SECTION_QUESTION, question);
 	ldns_pkt_set_aa(notify, true);
@@ -272,11 +324,10 @@ main(int argc, char **argv)
 		ldns_pkt_push_rr(notify, LDNS_SECTION_ANSWER, soa_rr);
 	}
 
-	if(tsig_cred.keyname) {
+	if(tsig_name && tsig_data) {
 #ifdef HAVE_SSL
-		status = ldns_pkt_tsig_sign(notify, tsig_cred.keyname,
-			tsig_cred.keydata, 300, tsig_cred.algorithm,
-			NULL);
+		status = ldns_pkt_tsig_sign(notify, tsig_name,
+			tsig_data, 300, tsig_algo, NULL);
 		if(status != LDNS_STATUS_OK) {
 			printf("Error TSIG sign query: %s\n",
 				ldns_get_errorstr_by_id(status));
@@ -293,11 +344,13 @@ main(int argc, char **argv)
 	}
 
 	status = ldns_pkt2wire(&wire, notify, &wiresize);
-	if(wiresize == 0) {
+	if (status) {
+		printf("Error converting notify packet to hex: %s\n",
+				ldns_get_errorstr_by_id(status));
+	} else if(wiresize == 0) {
 		printf("Error converting notify packet to hex.\n");
 		exit(1);
 	}
-
 	if(do_hexdump && verbose > 1) {
 		printf("Hexdump of notify packet:\n");
 		for(i=0; i<(int)wiresize; i++)
@@ -308,13 +361,13 @@ main(int argc, char **argv)
 	for(i=0; i<argc; i++)
 	{
 		struct addrinfo hints, *res0, *ai_res;
-		int error;
-		int default_family = AF_INET;
+		int default_family = AF_UNSPEC;
 
 		if(verbose)
 			printf("# sending to %s\n", argv[i]);
 		memset(&hints, 0, sizeof(hints));
 		hints.ai_family = default_family;
+		/* if(strchr(argv[i], ':')) hints.ai_family = AF_INET6; */
 		hints.ai_socktype = SOCK_DGRAM;
 		hints.ai_protocol = IPPROTO_UDP;
 		error = getaddrinfo(argv[i], port, &hints, &res0);
@@ -324,10 +377,19 @@ main(int argc, char **argv)
 			continue;
 		}
 		for (ai_res = res0; ai_res; ai_res = ai_res->ai_next) {
-			int s = socket(ai_res->ai_family, ai_res->ai_socktype, 
+			int s;
+
+			if (from0 && ai_res->ai_family != from0->ai_family)
+				continue;
+
+		        s = socket(ai_res->ai_family, ai_res->ai_socktype, 
 				ai_res->ai_protocol);
 			if(s == -1)
 				continue;
+			if (from0 && bind(s, from0->ai_addr, from0->ai_addrlen)) {
+				perror("Could not bind to source IP");
+				exit(EXIT_FAILURE);
+			}
 			/* send the notify */
 			notify_host(s, ai_res, wire, wiresize, argv[i]);
 		}
