@@ -23,6 +23,10 @@
 #include <ldns/buffer.h>
 #include <ldns/error.h>
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 /** Maximum length of a dname label */
 #define LDNS_MAX_LABELLEN     63
 /** Maximum length of a complete dname */
@@ -32,9 +36,6 @@
 /** The bytes TTL, CLASS and length use up in an rr */
 #define LDNS_RR_OVERHEAD	10
 
-/* The first fields are 'common' and can be referenced instantly */
-#define LDNS_RDATA_FIELD_DESCRIPTORS_COMMON 51
-
 
 
 /**
@@ -76,23 +77,23 @@ typedef enum ldns_enum_rr_compress ldns_rr_compress;
 enum ldns_enum_rr_type
 {
 	/**  a host address */
-	LDNS_RR_TYPE_A = 1, 
+	LDNS_RR_TYPE_A = 1,
 	/**  an authoritative name server */
-	LDNS_RR_TYPE_NS = 2, 
+	LDNS_RR_TYPE_NS = 2,
 	/**  a mail destination (Obsolete - use MX) */
-	LDNS_RR_TYPE_MD = 3, 
+	LDNS_RR_TYPE_MD = 3,
 	/**  a mail forwarder (Obsolete - use MX) */
-	LDNS_RR_TYPE_MF = 4, 
+	LDNS_RR_TYPE_MF = 4,
 	/**  the canonical name for an alias */
-	LDNS_RR_TYPE_CNAME = 5, 
+	LDNS_RR_TYPE_CNAME = 5,
 	/**  marks the start of a zone of authority */
-	LDNS_RR_TYPE_SOA = 6, 
+	LDNS_RR_TYPE_SOA = 6,
 	/**  a mailbox domain name (EXPERIMENTAL) */
-	LDNS_RR_TYPE_MB = 7, 
+	LDNS_RR_TYPE_MB = 7,
 	/**  a mail group member (EXPERIMENTAL) */
-	LDNS_RR_TYPE_MG = 8, 
+	LDNS_RR_TYPE_MG = 8,
 	/**  a mail rename domain name (EXPERIMENTAL) */
-	LDNS_RR_TYPE_MR = 9, 
+	LDNS_RR_TYPE_MR = 9,
 	/**  a null RR (EXPERIMENTAL) */
 	LDNS_RR_TYPE_NULL = 10,
 	/**  a well known service description */
@@ -155,33 +156,61 @@ enum ldns_enum_rr_type
 	LDNS_RR_TYPE_DNAME = 39,
 	/**  dnsind-kitchen-sink-02.txt */
 	LDNS_RR_TYPE_SINK = 40,
-	/**  Pseudo OPT record... */
+	/**  OPT record RFC 6891 */
 	LDNS_RR_TYPE_OPT = 41,
 	/**  RFC3123 */
 	LDNS_RR_TYPE_APL = 42,
-	/**  draft-ietf-dnsext-delegation */
+	/**  RFC4034, RFC3658 */
 	LDNS_RR_TYPE_DS = 43,
 	/**  SSH Key Fingerprint */
-	LDNS_RR_TYPE_SSHFP = 44,
-	/**  draft-richardson-ipseckey-rr-11.txt */
-	LDNS_RR_TYPE_IPSECKEY = 45,
-	/**  draft-ietf-dnsext-dnssec-25 */
-	LDNS_RR_TYPE_RRSIG = 46,
-	LDNS_RR_TYPE_NSEC = 47,      
-	LDNS_RR_TYPE_DNSKEY = 48,
-	LDNS_RR_TYPE_DHCID = 49,
+	LDNS_RR_TYPE_SSHFP = 44, /* RFC 4255 */
+	/**  IPsec Key */
+	LDNS_RR_TYPE_IPSECKEY = 45, /* RFC 4025 */
+	/**  DNSSEC */
+	LDNS_RR_TYPE_RRSIG = 46, /* RFC 4034 */
+	LDNS_RR_TYPE_NSEC = 47, /* RFC 4034 */
+	LDNS_RR_TYPE_DNSKEY = 48, /* RFC 4034 */
 
-        /* TODO: Temporary assume type code 50, 51 (from draft-11) */
-	LDNS_RR_TYPE_NSEC3 = 50,
+	LDNS_RR_TYPE_DHCID = 49, /* RFC 4701 */
+	/* NSEC3 */
+	LDNS_RR_TYPE_NSEC3 = 50, /* RFC 5155 */
+	LDNS_RR_TYPE_NSEC3PARAM = 51, /* RFC 5155 */
 	LDNS_RR_TYPE_NSEC3PARAMS = 51,
+	LDNS_RR_TYPE_TLSA = 52, /* RFC 6698 */
+	LDNS_RR_TYPE_SMIMEA = 53, /* RFC 8162 */
 
-	LDNS_RR_TYPE_SPF = 99,
+	LDNS_RR_TYPE_HIP = 55, /* RFC 5205 */
+
+	/** draft-reid-dnsext-zs */
+	LDNS_RR_TYPE_NINFO = 56,
+	/** draft-reid-dnsext-rkey */
+	LDNS_RR_TYPE_RKEY = 57,
+        /** draft-ietf-dnsop-trust-history */
+        LDNS_RR_TYPE_TALINK = 58,
+	LDNS_RR_TYPE_CDS = 59, /* RFC 7344 */
+	LDNS_RR_TYPE_CDNSKEY = 60, /* RFC 7344 */
+	LDNS_RR_TYPE_OPENPGPKEY = 61, /* RFC 7929 */
+	LDNS_RR_TYPE_CSYNC = 62, /* RFC 7477 */
+	LDNS_RR_TYPE_ZONEMD = 63, /* draft-ietf-dnsop-dns-zone-digest */
+	LDNS_RR_TYPE_SVCB = 64, /* draft-ietf-dnsop-svcb-https */
+	LDNS_RR_TYPE_HTTPS = 65, /* draft-ietf-dnsop-svcb-https */
+
+	LDNS_RR_TYPE_SPF = 99, /* RFC 4408 */
 
 	LDNS_RR_TYPE_UINFO = 100,
 	LDNS_RR_TYPE_UID = 101,
 	LDNS_RR_TYPE_GID = 102,
 	LDNS_RR_TYPE_UNSPEC = 103,
 
+	LDNS_RR_TYPE_NID = 104, /* RFC 6742 */
+	LDNS_RR_TYPE_L32 = 105, /* RFC 6742 */
+	LDNS_RR_TYPE_L64 = 106, /* RFC 6742 */
+	LDNS_RR_TYPE_LP = 107, /* RFC 6742 */
+
+	LDNS_RR_TYPE_EUI48 = 108, /* RFC 7043 */
+	LDNS_RR_TYPE_EUI64 = 109, /* RFC 7043 */
+
+	LDNS_RR_TYPE_TKEY = 249, /* RFC 2930 */
 	LDNS_RR_TYPE_TSIG = 250,
 	LDNS_RR_TYPE_IXFR = 251,
 	LDNS_RR_TYPE_AXFR = 252,
@@ -191,7 +220,16 @@ enum ldns_enum_rr_type
 	LDNS_RR_TYPE_MAILA = 254,
 	/**  any type (wildcard) */
 	LDNS_RR_TYPE_ANY = 255,
+	LDNS_RR_TYPE_URI = 256, /* RFC 7553 */
+	LDNS_RR_TYPE_CAA = 257, /* RFC 6844 */
+	LDNS_RR_TYPE_AVC = 258, /* Cisco's DNS-AS RR, see www.dns-as.org */
+	LDNS_RR_TYPE_DOA = 259, /* draft-durand-doa-over-dns */
 
+	/** draft-ietf-mboned-driad-amt-discovery **/
+	LDNS_RR_TYPE_AMTRELAY = 260,
+
+	/** DNSSEC Trust Authorities */
+	LDNS_RR_TYPE_TA = 32768,
 	/* RFC 4431, 5074, DNSSEC Lookaside Validation */
 	LDNS_RR_TYPE_DLV = 32769,
 
@@ -204,6 +242,9 @@ enum ldns_enum_rr_type
 };
 typedef enum ldns_enum_rr_type ldns_rr_type;
 
+/* The first fields are contiguous and can be referenced instantly */
+#define LDNS_RDATA_FIELD_DESCRIPTORS_COMMON (LDNS_RR_TYPE_AMTRELAY + 1)
+
 /**
  * Resource Record
  *
@@ -268,18 +309,22 @@ RDATA           a variable length string of octets that describes the
 struct ldns_struct_rr
 {
 	/**  Owner name, uncompressed */
-	ldns_rdf	*_owner;	
+	ldns_rdf	*_owner;
 	/**  Time to live  */
-	uint32_t	_ttl;	
+	uint32_t	_ttl;
 	/**  Number of data fields */
 	size_t	        _rd_count;
 	/**  the type of the RR. A, MX etc. */
-	ldns_rr_type	_rr_type;	
+	ldns_rr_type	_rr_type;
 	/**  Class of the resource record.  */
 	ldns_rr_class	_rr_class;
 	/* everything in the rdata is in network order */
 	/**  The array of rdata's */
 	ldns_rdf	 **_rdata_fields;
+	/**  question rr [it would be nicer if thous is after _rd_count]
+		 ABI change: Fix this in next major release
+	 */
+	bool		_rr_question;
 };
 typedef struct ldns_struct_rr ldns_rr;
 
@@ -287,7 +332,7 @@ typedef struct ldns_struct_rr ldns_rr;
  * List or Set of Resource Records
  *
  * Contains a list of rr's <br>
- * No official RFC-like checks are made 
+ * No official RFC-like checks are made
  */
 struct ldns_struct_rr_list
 {
@@ -323,13 +368,30 @@ struct ldns_struct_rr_descriptor
 };
 typedef struct ldns_struct_rr_descriptor ldns_rr_descriptor;
 
-/**     
+
+/**
+ * Create a rr type bitmap rdf providing enough space to set all 
+ * known (to ldns) rr types.
+ * \param[out] rdf the constructed rdf
+ * \return LDNS_STATUS_OK if all went well.
+ */
+ldns_status ldns_rdf_bitmap_known_rr_types_space(ldns_rdf** rdf);
+
+/**
+ * Create a rr type bitmap rdf with at least all known (to ldns) rr types set.
+ * \param[out] rdf the constructed rdf
+ * \return LDNS_STATUS_OK if all went well.
+ */
+ldns_status ldns_rdf_bitmap_known_rr_types(ldns_rdf** rdf);
+
+
+/**
  * creates a new rr structure.
  * \return ldns_rr *
  */
 ldns_rr* ldns_rr_new(void);
 
-/** 
+/**
  * creates a new rr structure, based on the given type.
  * alloc enough space to hold all the rdf's
  */
@@ -337,7 +399,7 @@ ldns_rr* ldns_rr_new_frm_type(ldns_rr_type t);
 
 /**
  * frees an RR structure
- * \param[in] *rr the RR to be freed 
+ * \param[in] *rr the RR to be freed
  * \return void
  */
 void ldns_rr_free(ldns_rr *rr);
@@ -345,13 +407,13 @@ void ldns_rr_free(ldns_rr *rr);
 /**
  * creates an rr from a string.
  * The string should be a fully filled-in rr, like
- * ownername &lt;space&gt; TTL &lt;space&gt; CLASS &lt;space&gt; 
+ * ownername &lt;space&gt; TTL &lt;space&gt; CLASS &lt;space&gt;
  * TYPE &lt;space&gt; RDATA.
  * \param[out] n the rr to return
  * \param[in] str the string to convert
  * \param[in] default_ttl default ttl value for the rr.
  *            If 0 DEF_TTL will be used
- * \param[in] origin when the owner is relative add this. 
+ * \param[in] origin when the owner is relative add this.
  *	The caller must ldns_rdf_deep_free it.
  * \param[out] prev the previous ownername. if this value is not NULL,
  * the function overwrites this with the ownername found in this
@@ -359,7 +421,7 @@ void ldns_rr_free(ldns_rr *rr);
  * \return a status msg describing an error or LDNS_STATUS_OK
  */
 ldns_status ldns_rr_new_frm_str(ldns_rr **n, const char *str,
-                                uint32_t default_ttl, ldns_rdf *origin,
+                                uint32_t default_ttl, const ldns_rdf *origin,
                                 ldns_rdf **prev);
 
 /**
@@ -369,14 +431,14 @@ ldns_status ldns_rr_new_frm_str(ldns_rr **n, const char *str,
  * ldns_rr_new_frm_str()
  * \param[out] n the rr to return
  * \param[in] str the string to convert
- * \param[in] origin when the owner is relative add this. 
+ * \param[in] origin when the owner is relative add this.
  *	The caller must ldns_rdf_deep_free it.
  * \param prev the previous ownername. the function overwrite this with
  * the current found ownername. The caller must ldns_rdf_deep_free it.
  * \return a status msg describing an error or LDNS_STATUS_OK
  */
 ldns_status ldns_rr_new_question_frm_str(ldns_rr **n, const char *str,
-                                ldns_rdf *origin, ldns_rdf **prev);
+                                const ldns_rdf *origin, ldns_rdf **prev);
 
 /**
  * creates a new rr from a file containing a string.
@@ -419,6 +481,14 @@ ldns_status ldns_rr_new_frm_fp_l(ldns_rr **rr, FILE *fp, uint32_t *default_ttl,
  */
 void ldns_rr_set_owner(ldns_rr *rr, ldns_rdf *owner);
 
+/**
+ * sets the question flag in the rr structure.
+ * \param[in] *rr rr to operate on
+ * \param[in] question question flag
+ * \return void
+ */
+void ldns_rr_set_question(ldns_rr *rr, bool question);
+
 /**
  * sets the ttl in the rr structure.
  * \param[in] *rr rr to operate on
@@ -452,17 +522,17 @@ void ldns_rr_set_type(ldns_rr *rr, ldns_rr_type rr_type);
 void ldns_rr_set_class(ldns_rr *rr, ldns_rr_class rr_class);
 
 /**
- * sets a rdf member, it will be set on the 
+ * sets a rdf member, it will be set on the
  * position given. The old value is returned, like pop.
  * \param[in] *rr the rr to operate on
  * \param[in] *f the rdf to set
  * \param[in] position the position the set the rdf
- * \return  the old value in the rr, NULL on failyre
+ * \return  the old value in the rr, NULL on failure
  */
 ldns_rdf* ldns_rr_set_rdf(ldns_rr *rr, const ldns_rdf *f, size_t position);
 
 /**
- * sets rd_field member, it will be 
+ * sets rd_field member, it will be
  * placed in the next available spot.
  * \param[in] *rr rr to operate on
  * \param[in] *f the data field member to set
@@ -471,7 +541,7 @@ ldns_rdf* ldns_rr_set_rdf(ldns_rr *rr, const ldns_rdf *f, size_t position);
 bool ldns_rr_push_rdf(ldns_rr *rr, const ldns_rdf *f);
 
 /**
- * removes a rd_field member, it will be 
+ * removes a rd_field member, it will be
  * popped from the last position.
  * \param[in] *rr rr to operate on
  * \return rdf which was popped (null if nothing)
@@ -489,10 +559,17 @@ ldns_rdf* ldns_rr_rdf(const ldns_rr *rr, size_t nr);
 /**
  * returns the owner name of an rr structure.
  * \param[in] *rr rr to operate on
- * \return ldns_rdf * 
+ * \return ldns_rdf *
  */
 ldns_rdf* ldns_rr_owner(const ldns_rr *rr);
 
+/**
+ * returns the question flag of an rr structure.
+ * \param[in] *rr rr to operate on
+ * \return bool true if question
+ */
+bool ldns_rr_is_question(const ldns_rr *rr);
+
 /**
  * returns the ttl of an rr structure.
  * \param[in] *rr the rr to read from
@@ -560,7 +637,7 @@ ldns_rr* ldns_rr_list_rr(const ldns_rr_list *rr_list, size_t nr);
  * creates a new rr_list structure.
  * \return a new rr_list structure
  */
-ldns_rr_list* ldns_rr_list_new();
+ldns_rr_list* ldns_rr_list_new(void);
 
 /**
  * frees an rr_list structure.
@@ -581,7 +658,7 @@ void ldns_rr_list_deep_free(ldns_rr_list *rr_list);
  * \param[in] right the rightside
  * \return a left with right concatenated to it
  */
-bool ldns_rr_list_cat(ldns_rr_list *left, ldns_rr_list *right);
+bool ldns_rr_list_cat(ldns_rr_list *left, const ldns_rr_list *right);
 
 /**
  * concatenates two ldns_rr_lists together, but makes clones of the rr's 
@@ -590,7 +667,7 @@ bool ldns_rr_list_cat(ldns_rr_list *left, ldns_rr_list *right);
  * \param[in] right the rightside
  * \return a new rr_list with leftside/rightside concatenated
  */
-ldns_rr_list* ldns_rr_list_cat_clone(ldns_rr_list *left, ldns_rr_list *right);
+ldns_rr_list* ldns_rr_list_cat_clone(const ldns_rr_list *left, const ldns_rr_list *right);
 
 /**
  * pushes an rr to an rrlist.
@@ -630,14 +707,14 @@ ldns_rr_list* ldns_rr_list_pop_rr_list(ldns_rr_list *rr_list, size_t size);
  * \param[in] rr the rr to check
  * \return true if rr_list contains rr, false otherwise
  */
-bool ldns_rr_list_contains_rr(const ldns_rr_list *rr_list, ldns_rr *rr); 
+bool ldns_rr_list_contains_rr(const ldns_rr_list *rr_list, const ldns_rr *rr); 
 
 /**
  * checks if an rr_list is a rrset.
  * \param[in] rr_list the rr_list to check
  * \return true if it is an rrset otherwise false
  */
-bool ldns_is_rrset(ldns_rr_list *rr_list);
+bool ldns_is_rrset(const ldns_rr_list *rr_list);
 
 /**
  * pushes an rr to an rrset (which really are rr_list's).
@@ -708,7 +785,6 @@ void ldns_rr_list_sort(ldns_rr_list *unsorted);
  *         +1 if rr2 comes before rr1
  */
 int ldns_rr_compare(const ldns_rr *rr1, const ldns_rr *rr2);
-int ldns_rr_compare_nsec3(const ldns_rr *rr1, const ldns_rr *rr2);
 
 /**
  * compares two rrs, up to the rdata.
@@ -728,7 +804,7 @@ int ldns_rr_compare_no_rdata(const ldns_rr *rr1, const ldns_rr *rr2);
  *         -1 if rr1_buf comes before rr2_buf
  *         +1 if rr2_buf comes before rr1_buf
  */
-int ldns_rr_compare_wire(ldns_buffer *rr1_buf, ldns_buffer *rr2_buf);
+int ldns_rr_compare_wire(const ldns_buffer *rr1_buf, const ldns_buffer *rr2_buf);
 
 /**
  * returns true of the given rr's are equal.
@@ -741,7 +817,7 @@ int ldns_rr_compare_wire(ldns_buffer *rr1_buf, ldns_buffer *rr2_buf);
 bool ldns_rr_compare_ds(const ldns_rr *rr1, const ldns_rr *rr2);
 
 /**
- * compares two rr listss.
+ * compares two rr lists.
  * \param[in] rrl1 the first one
  * \param[in] rrl2 the second one
  * \return 0 if equal
@@ -769,14 +845,14 @@ void ldns_rr2canonical(ldns_rr *rr);
  * \param[in] rr_list the rr_list to work on
  * \return void
  */
-void ldns_rr_list2canonical(ldns_rr_list *rr_list);
+void ldns_rr_list2canonical(const ldns_rr_list *rr_list);
 
 /** 
  * counts the number of labels of the ownername.
  * \param[in] rr count the labels of this rr
  * \return the number of labels
  */
-uint8_t ldns_rr_label_count(ldns_rr *rr);
+uint8_t ldns_rr_label_count(const ldns_rr *rr);
 
 /**
  * returns the resource record descriptor for the given rr type.
@@ -822,11 +898,11 @@ ldns_rdf_type ldns_rr_descriptor_field_type(const ldns_rr_descriptor *descriptor
  * \return a new rr list with only the RRs that match 
  *
  */
-ldns_rr_list *ldns_rr_list_subtype_by_rdf(ldns_rr_list *l, ldns_rdf *r, size_t pos);
+ldns_rr_list *ldns_rr_list_subtype_by_rdf(const ldns_rr_list *l, const ldns_rdf *r, size_t pos);
 
 /**
  * convert an rdf of type LDNS_RDF_TYPE_TYPE to an actual
- * LDNS_RR_TYPE. This is usefull in the case when inspecting
+ * LDNS_RR_TYPE. This is useful in the case when inspecting
  * the rrtype covered field of an RRSIG.
  * \param[in] rd the rdf to look at
  * \return a ldns_rr_type with equivalent LDNS_RR_TYPE
@@ -854,4 +930,8 @@ ldns_rr_list_type(const ldns_rr_list *rr_list);
 ldns_rdf *
 ldns_rr_list_owner(const ldns_rr_list *rr_list);
 
+#ifdef __cplusplus
+}
+#endif
+
 #endif /* LDNS_RR_H */
